package thread;

import domain.Manufacturer;
import domain.Product;
import domain.User;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import service.ServiceManufacturer;
import service.ServiceProduct;
import service.ServiceUser;
import service.impl.ServiceManufacturerImpl;
import service.impl.ServiceProductImpl;
import service.impl.ServiceUserImpl;
import transfer.RequestObject;
import transfer.ResponseObject;
import util.Operation;
import util.ResponseStatus;

/**
 *
 * @author Dusan
 */
public class ClientThread extends Thread {

    private final Socket socket;
    private final ObjectInputStream objectInputStream;
    private final ObjectOutputStream objectOutputStream;

    private final ServiceUser serviceUser;
    private final ServiceManufacturer serviceManufacturer;
    private final ServiceProduct serviceProduct;
    
    private User loginUser;
    
    public ClientThread(Socket socket) throws IOException {
        this.socket = socket;
        objectInputStream = new ObjectInputStream(socket.getInputStream());
        objectOutputStream = new ObjectOutputStream(socket.getOutputStream());
        serviceUser = new ServiceUserImpl();
        serviceManufacturer = new ServiceManufacturerImpl();
        serviceProduct=new ServiceProductImpl();
    }

    @Override
    public void run() {
        while (!socket.isClosed()) {
            try {
                RequestObject requestObject = (RequestObject) objectInputStream.readObject();
                ResponseObject responseObject = handleRequest(requestObject);
                objectOutputStream.writeObject(responseObject);
            } catch (IOException | ClassNotFoundException ex) {
                System.out.println(ex.getMessage());
            }
        }
    }

    private ResponseObject handleRequest(RequestObject requestObject) {
        int operation = requestObject.getOperation();
        switch (operation) {
            case Operation.OPERATION_LOGIN:
                return login((Map) requestObject.getData());
            case Operation.OPERATION_ALL_MANUFACTURES:
                return getAllManufactures();
            case Operation.OPERATION_SAVE_PRODUCT:
                return saveProduct((Product)requestObject.getData());
        }
        return null;
    }

    private ResponseObject login(Map data) {
        String username = (String) data.get("username");
        String password = (String) data.get("password");
        ResponseObject responseObject = new ResponseObject();
        try {
            User user = serviceUser.login(username, password);
            responseObject.setData(user);
            responseObject.setStatus(ResponseStatus.SUCCESS);
            loginUser=user;
        } catch (Exception ex) {
            responseObject.setStatus(ResponseStatus.ERROR);
            responseObject.setErrorMessage(ex.getMessage());
        }

        return responseObject;
    }

    public Socket getSocket() {
        return socket;
    }

    private ResponseObject getAllManufactures() {
         ResponseObject responseObject = new ResponseObject();
        try {
           List<Manufacturer> manufactures = serviceManufacturer.getAll();
            responseObject.setData(manufactures);
            responseObject.setStatus(ResponseStatus.SUCCESS);
        } catch (Exception ex) {
            responseObject.setStatus(ResponseStatus.ERROR);
            responseObject.setErrorMessage(ex.getMessage());
        }

        return responseObject;
        
    }

    private ResponseObject saveProduct(Product product) {
        //servis za proizvod
        ResponseObject responseObject=new ResponseObject();
        try {
            Product p = serviceProduct.save(product);
            responseObject.setStatus(ResponseStatus.SUCCESS);
            responseObject.setData(p);
            
        } catch (Exception ex) {
            ex.printStackTrace();
            responseObject.setStatus(ResponseStatus.ERROR);
            responseObject.setErrorMessage(ex.getMessage());
        }
        return responseObject;
    }

    public User getLoginUser() {
        return loginUser;
    }

    
}
